@@ -138,38 +138,60 @@ where
     }
 
     #[emitter]
-    fn emit_export_decl(&mut self, node: &ExportDecl) -> Result {
-        keyword!("export");
+    fn emit_export_decl(&mut self, n: &ExportDecl) -> Result {
+        {
+            let span = if n.span.is_dummy() {
+                DUMMY_SP
+            } else {
+                Span::new(n.span.lo, n.span.lo + BytePos(6), Default::default())
+            };
+            keyword!(span, "export");
+        }
         space!();
-        emit!(node.decl);
+        emit!(n.decl);
     }
 
     #[emitter]
-    fn emit_export_default_expr(&mut self, node: &ExportDefaultExpr) -> Result {
-        keyword!("export");
+    fn emit_export_default_expr(&mut self, n: &ExportDefaultExpr) -> Result {
+        {
+            let span = if n.span.is_dummy() {
+                DUMMY_SP
+            } else {
+                Span::new(n.span.lo, n.span.lo + BytePos(6), Default::default())
+            };
+            keyword!(span, "export");
+        }
+
         space!();
         keyword!("default");
         {
-            let starts_with_alpha_num = node.expr.starts_with_alpha_num();
+            let starts_with_alpha_num = n.expr.starts_with_alpha_num();
             if starts_with_alpha_num {
                 space!();
             } else {
                 formatting_space!();
             }
-            emit!(node.expr);
+            emit!(n.expr);
         }
         formatting_semi!();
     }
 
     #[emitter]
-    fn emit_export_default_decl(&mut self, node: &ExportDefaultDecl) -> Result {
-        self.emit_leading_comments_of_span(node.span(), false)?;
+    fn emit_export_default_decl(&mut self, n: &ExportDefaultDecl) -> Result {
+        self.emit_leading_comments_of_span(n.span(), false)?;
 
-        keyword!("export");
+        {
+            let span = if n.span.is_dummy() {
+                DUMMY_SP
+            } else {
+                Span::new(n.span.lo, n.span.lo + BytePos(6), Default::default())
+            };
+            keyword!(span, "export");
+        }
         space!();
         keyword!("default");
         space!();
-        match node.decl {
+        match n.decl {
             DefaultDecl::Class(ref n) => emit!(n),
             DefaultDecl::Fn(ref n) => emit!(n),
             DefaultDecl::TsInterfaceDecl(ref n) => emit!(n),
@@ -347,7 +369,15 @@ where
             },
         );
 
-        keyword!("export");
+        {
+            let span = if node.span.is_dummy() {
+                DUMMY_SP
+            } else {
+                Span::new(node.span.lo, node.span.lo + BytePos(6), Default::default())
+            };
+            keyword!(span, "export");
+        }
+
         formatting_space!();
         if let Some(spec) = namespace_spec {
             emit!(spec);
@@ -856,12 +886,7 @@ where
                         }),
                     ) => false,
 
-                    (_, Expr::Update(UpdateExpr { prefix: true, .. }) | Expr::Unary(..)) => true,
-
-                    (_, Expr::Bin(BinExpr { left, .. })) => match &**left {
-                        Expr::Update(UpdateExpr { prefix: true, .. }) | Expr::Unary(..) => true,
-                        _ => false,
-                    },
+                    (_, r) if is_space_require_before_rhs(r) => true,
 
                     _ => false,
                 }
@@ -2799,7 +2824,7 @@ fn should_emit_whitespace_before_operand(node: &UnaryExpr) -> bool {
         _ => {}
     }
 
-    match *node.arg {
+    match &*node.arg {
         Expr::Update(UpdateExpr {
             op: op!("++"),
             prefix: true,
@@ -2818,6 +2843,9 @@ fn should_emit_whitespace_before_operand(node: &UnaryExpr) -> bool {
             op: op!(unary, "-"),
             ..
         }) if node.op == op!(unary, "-") => true,
+
+        Expr::Lit(Lit::Num(v)) if v.value.is_sign_negative() && node.op == op!(unary, "-") => true,
+
         _ => false,
     }
 }
@@ -3201,3 +3229,15 @@ fn handle_invalid_unicodes(s: &str) -> Cow<str> {
 
     Cow::Owned(s.replace("\\\0", "\\"))
 }
+
+fn is_space_require_before_rhs(rhs: &Expr) -> bool {
+    match rhs {
+        Expr::Lit(Lit::Num(v)) if v.value.is_sign_negative() => true,
+
+        Expr::Update(UpdateExpr { prefix: true, .. }) | Expr::Unary(..) => true,
+
+        Expr::Bin(BinExpr { left, .. }) => is_space_require_before_rhs(&left),
+
+        _ => false,
+    }
+}
